from datetime import datetime, timedelta
from rsm.override.rsmsdk import RsmSDK
from rsm.handlers.output import LogHandler
from dateutil import parser
import pytz
import os

TIMESTAMP_FILE = "timestamp.stmp"
fmt = '%Y-%m-%d %H:%M:%S %Z%z'
utc = pytz.timezone('UTC')
eastern = pytz.timezone('US/Eastern')

logger = LogHandler(__name__)
debug = logger.debug
info = logger.info
error = logger.error

# HELP YOURSELF BY READING THIS.
#
# There is only one unix time and it is created by using UTC/GMT time zone.
# This means time should always be handled by timestamps and can be presented in human readable format
# but it will only be converted for this purpose.
#
# In short, any logic should use timestamps.


# The preferred way of dealing with times is to always work in UTC,
# converting to localtime only when generating output to be read by humans.

# >>> utc_dt = datetime(2002, 10, 27, 6, 0, 0, tzinfo=utc)
# >>> loc_dt = utc_dt.astimezone(eastern)
# >>> loc_dt.strftime(fmt)
# '2002-10-27 01:00:00 EST-0500'

# Creating local times is also tricky, and the reason why working with local times is not recommended.
# Unfortunately, you cannot just pass a tzinfo argument when constructing a datetime

# source: http://pytz.sourceforge.net/


def convert_datetime_to_unix_time(dt: datetime, ms=True) -> int:
    """
    returns the time in unix time
    :param ms:
    :param dt: {datetime}
    :return: {unix time}
    """
    return int(dt.timestamp()) * 1000 if ms else int(dt.timestamp())


def from_utc_to_timezone(time: datetime or str or int, timezone: pytz.timezone = None) \
        -> datetime or int or str:
    """

    :param time: UTC time
    :param timezone: timezone to convert time to
    :return: converted datetime
    """

    timezone = pytz.timezone(timezone)
    if isinstance(time, str):
        debug('Variable `time` is type < str >, removing tzinfo and parsing datetime')
        time = parser.parse(time).timestamp()

    if isinstance(time, (int, float)):
        debug('Variable `time` is type < int >, removing tzinfo and parsing datetime')
        if isinstance(time, float):
            time = int(time)

        try:
            if len(str(int(time))) == 10:  # timestamp in seconds
                time = time * 1000

            return datetime.fromtimestamp(int(time.timestamp())).astimezone(timezone)
        except OverflowError as err:
            error(err)

    # Converting to localtime ONLY when generating output to be read by humans.
    return time.astimezone(timezone)


def convert_string_to_datetime(datetime_str: str) \
        -> datetime:
    """
    returns time in datetime format
    """
    try:

        # This does NOT convert timezone, it defines the TIMEZONE of this datetime.
        # Example:
        #
        # >>> dt = 2020-05-05 00:00:00+00:00
        # >>> timezone = pytz.timezone("US/Eastern")
        # >>> dt = dt.astimezone(timezone)
        #
        # returns ---> 2020-05-05 00:00:00-04:00

        return parser.parse(datetime_str)

    except Exception as e:
        raise Exception(
            "{0}: {1}".format(
                "convert_string_to_datetime Failed",
                str(e)))  # TODO: Why raise again?


def convert_unixtime_to_datetime(unix_time: int) -> datetime:
    """
    returns the time in local time with stated TZ
    """
    try:
        return datetime.fromtimestamp(unix_time)
    except Exception as e:
        raise Exception(
            "{0}: {1}".format(
                "convert_unixtime_to_datetime Failed",
                str(e)))  # TODO: Why raise again?


def convert_string_to_unix_time(datetime_str: str = None) -> int:
    """
    return time in unix time format
    """
    try:
        dt = convert_string_to_datetime(datetime_str)
        return convert_datetime_to_unix_time(dt, True)
    except Exception as e:
        raise Exception(
            "{0}: {1}".format(
                "convert_string_to_unix_time Failed",
                str(e)))  # TODO: Why raise again?


def validate_timestamp(last_run_timestamp: datetime, offset: int) -> datetime:
    """
    Validate timestamp in range
    """

    if isinstance(last_run_timestamp, str):
        last_run_timestamp = datetime.strptime(last_run_timestamp, fmt)

    current_time = datetime.utcnow()
    if current_time - last_run_timestamp < timedelta(days=offset):
        return current_time - timedelta(days=offset)
    else:
        return last_run_timestamp


def fetch_timestamp(datetime_format=False):
    """
    get timestamp
    :param datetime_format: {boolean} if datetime - return timestamp as datetime
    :return: {unix time/ datetime}
    """
    rsm_sdk_config = RsmSDK()
    save_file_path = os.path.join(rsm_sdk_config.run_folder_path, TIMESTAMP_FILE)

    last_run_time = 0
    if os.path.isfile(save_file_path):
        with open(save_file_path, 'r') as f:
            last_run_time = f.read()
            f.close()

    try:
        last_run_time = int(last_run_time)
    except BaseException:
        last_run_time = convert_string_to_unix_time(last_run_time)

    if datetime_format:
        last_run_time = convert_unixtime_to_datetime(last_run_time)

    else:
        last_run_time = int(last_run_time)

    return last_run_time



import getopt, sys
from rsm.override.rsmsdk import RsmSDK as SiemplifySdkConfig


class SiemplifyBase:

	def __init__(self):
		def __init__(self):
			self.sdk_config = SiemplifySdkConfig()
			self.RUN_FOLDER = self.sdk_config.run_folder_path
			self.script_name = ""
			self._logger = None
			self._logs_collector = None
			self._log_path = None
			self._log_use_elastic = False
			options, _ = getopt.gnu_getopt(sys.argv[1:], "", ["useElastic", "logPath="])

			for name, value in options:
				if name == "--logPath":
					self._log_path = value.strip('"')
				elif name == "--useElastic":
					self._log_use_elastic = True




from rsm.handlers.output import LogHandler
from rsm.api.core import ApiRequest
import pandas as pd
import numpy as np
import json

if __name__ == '__main__':
	logger = LogHandler('rsm.api.integrations')
else:
	logger = LogHandler(__name__)

log = logger.logger


def integration_instances(environment, integration):
	"""
	Integration's instances for the chosen environments
	:return: instances
	"""
	try:
		if not isinstance(integration, str):
			raise TypeError(
				"Parameter: integration",
				"Wrong type: {0}".format(
					type(
						integration)))
		elif isinstance(environment, str):
			log.warning("The variable 'environment' is a string, splitting by commas")
			environment = environment.split(',')
		elif not isinstance(environment, list):
			raise TypeError(
				"Parameter: environment",
				"Wrong type: {0}".format(
					type(environment)))
		endpoint = '/external/v1/integrations/GetOptionalIntegrationInstances'
		payload = json.dumps(
			{
				"environments":
					[env for env in environment]
					if environment is not None else [""],
				"integrationIdentifier": "{0}".format(integration)
			}
		)

		result = ApiRequest().make("POST", endpoint, data=payload)

		if isinstance(result, list):
			if len(result) > 1:
				log.info(
					f"Found {len(result)} instances for environment {environment} and the '{integration}' integration")
				return result
			elif len(result) == 0:
				log.debug(f'No instances were found for environment {environment} and the {integration} integration')
				return False
		else:
			return result
	except Exception as err:
		if isinstance(err.args, tuple) and len(err.args) == 1:
			log.error(err.args[0])
		else:
			log.error(err.args)


def get_installed_integrations():
	"""
	:return Returns a dataframe containing the installed integrations.
	"""

	endpoint = '/external/v1/integrations/GetInstalledIntegrations'
	rq = ApiRequest().make('GET', endpoint)

	integrations = [integration for integration in rq]
	# actions = [action['integrationSupportedActions'] for action in integrations]
	integrations = pd.DataFrame(data=[
		np.array([key['displayName'], key['version'], key['installedVersion'],
				[action['name'] for action in key['integrationSupportedActions']],
				key['description']], dtype=object)
		for key in integrations],
		columns=['name', 'version', 'installed_version', 'actions', 'description'])
	return integrations



from typing import NoReturn
from pathlib import Path
import getpass
import os


class CustomDataStore:
    def __call__(self, integration, script_name) -> NoReturn:
        self.integration = integration
        self.script_name = script_name

        # Not using Path.home() to prevent getting fooled by environment variable.

        self.basedir = Path(
            os.path.expanduser(
                '~{0}'.format(
                    getpass.getuser()))) / 'custom'

        return self.__init_custom_basedir(), self.__init_integration_folder()

    def __init_integration_folder(self) -> NoReturn or Exception:
        n = [self.integration, self.script_name]
        try:
            for idx, path in enumerate(n):
                previous = "/".join(n[:idx]) if idx > 0 else ''
                self.__is_dir(self.basedir / previous / path)
        except Exception as err:
            return err
        return self.__is_dir(
            self.basedir /
            self.integration /
            self.script_name)

    def __init_custom_basedir(self) -> NoReturn:
        return self.__is_dir(self.basedir)

    @property
    def ds_path(self) -> Path:
        return self.basedir / self.integration / self.script_name

    @staticmethod
    def __is_dir(dirpath: Path) -> NoReturn:
        dirpath.mkdir(mode=0o775, exist_ok=True)



from collections.abc import MutableMapping
import importlib.util
import functools
import inspect
import time


def add_method(cls):
	"""
	Add method to cls
	:param cls: Class
	:return:
	"""
	def decorator(func):
		@functools.wraps(func)
		def wrapper(self, *args, **kwargs):
			return func(*args, **kwargs)
		setattr(cls, func.__name__, wrapper)
		# Note we are not binding func, but wrapper which accepts self but does exactly the same as func
		return func # returning func means func can still be used normally
	return decorator


def get_class_that_defined_method(meth):
	"""
	Get defining class of method
	source: https://stackoverflow.com/a/25959545
	:param meth:
	:return: Example: <class '__main__.RsmJob'>
	"""
	if inspect.ismethod(meth):
		for cls in inspect.getmro(meth.__self__.__class__):
			if meth.__name__ in cls.__dict__:
				return cls
		meth = meth.__func__  # fallback to __qualname__ parsing
	if inspect.isfunction(meth):
		cls = getattr(
			inspect.getmodule(meth),
			meth.__qualname__.split(
				'.<locals>',
				1)[0].rsplit(
				'.',
				1)[0],
			None)
		if isinstance(cls, type):
			return cls
	return None


def get_module_location(mod):
	"""
	List of strings for where to find submodules, if a package (None otherwise).
	:param mod: Module to find the specs.
	:return: submodule_search_locations
	"""
	spec = importlib.util.find_spec(mod)
	return spec.submodule_search_locations


def benchmark(func):
	"""Print the runtime of the decorated function"""

	@functools.wraps(func)
	def wrapper_timer(*args, **kwargs):
		start_time = time.perf_counter()
		value = func(*args, **kwargs)
		end_time = time.perf_counter()
		run_time = end_time - start_time
		print(f"Finished {func.__name__!r} in {run_time:.4f} secs")
		return value

	return wrapper_timer  # no "()" here, we need the object to be returned.


def dict_to_flat(
		dictionary: dict or MutableMapping,
		parent_key: bool = False,
		separator: str = '.') -> dict:
	"""
	Turn a nested dictionary into a flattened dictionary
	:param dictionary: The dictionary to flatten
	:param parent_key: The string to prepend to dictionary's keys
	:param separator: The string used to separate flattened keys
	:return: A flattened dictionary
	"""

	items = []
	for key, value in dictionary.items():
		new_key = str(parent_key) + separator + key if parent_key else key
		if isinstance(value, MutableMapping):
			items.extend(dict_to_flat(value, new_key, separator).items())
		elif isinstance(value, list):
			for k, v in enumerate(value):
				items.extend(dict_to_flat({str(k): v}, new_key).items())
		else:
			items.append((new_key, value))
	return dict(items)


import os
import re
import json
import requests

from rsm.override.rsmsdk import RsmSDK
from rsm.mocking.ScriptResult import ScriptResult
from rsm.mocking.SiemplifyBase import SiemplifyBase

REGEX_CONFIGURATION = r"(?<=\[)[^[\]\:]*:::[^[\]]*(?=])"
EXTERNAL_CONFIG_PROVIDER_FILE = r"external_providers.json"
INSIGHT_DEFAULT_THREAT_SOURCE = "Siemplify System"
REMOTE_INTEGRATION_FILE = 'integration_conf.json'
EXTERNAL_PROVIDER_SEPARATOR = ":::"
HEADERS = {'Content-Type': 'application/json', 'Accept': 'application/json'}


class Siemplify(SiemplifyBase):
    def __init__(self):
        super(Siemplify, self).__init__()
        self.api_key = os.environ["AppKey"]  # original: self.api_key = sys.argv[1]
        self._result = ScriptResult([])
        self.API_ROOT = self.sdk_config.api_root_uri
        self.is_remote = self.sdk_config.is_remote_publisher_sdk

        if not self.is_remote:
            # Create regular Session
            self.session = requests.Session()
            self.session.verify = False
            HEADERS.update({"AppKey": self.api_key})
            self.session.headers.update(HEADERS)


from unittest import TestCase
from rsm.utils import misc

d = {
    "firstName": "John",
    "lastName": "Doe",
    "age": "27",
    "hobby": ["Tennis", "Volleyball", "Foo"],
    "attrib": {
        "stamina": 80,
        "dexterity": 15,
        "intelligence": 9000,
        "wisdom": 50
    }
}


class TestMisc(TestCase):
    def test_flatten(self):
        flat = misc.dict_to_flat(d)
        print(flat)
        assert flat



from rsm.handlers.output import LogHandler
import rsm.api.integrations

log = LogHandler(__name__)


class ParseIntegrations:
	def __init__(self):
		self.integrations = rsm.api.integrations.get_installed_integrations()

	def parsed_installed_integration(self):
		"""
		# TODO: include the description of each action (tuple)
		:return: JSON results containing the Integration, Actions and Description
		"""
		integrations = self.integrations
		result = {}
		for integration in integrations.name.tolist():
			selected = integrations[integrations['name'] == integration]
			if integration not in result.keys():
				result.update({integration: {}})
			integrations.explode('actions').dropna(subset=['actions'])
			result[integration] = {
				"Actions": selected.explode('actions')['actions'].dropna().tolist(),
				"Description": selected.description.max(),

				# The parameters below are commented since the API endpoint use for this request does not return
				# "InstalledVersion", "hasConnector" and many other fields properly.
				# TODO: change for this endpoint: /v1/store/GetIntegrationsStoreData
				# "Version": selected.version.max(),
				# "InstalledVersion": selected.installed_version.max()
			}
		return result

	def get_installed_integration_actions(self, name):
		"""

		:param name: Integration name
		:return: Dataframe containing all the available actions of the chosen integration.
		"""
		df = self.integrations
		return df['actions'][df['name'] == name].explode('actions')






from rsm.utils.misc import get_module_location
from platform import system
import logging.config
import functools
import logging


RSM_MODULE_LOCATION = get_module_location('rsm')[0]

DEFAULT_LOGGING_CONFIG_FILENAME = "logging.conf"
DEFAULT_LOGGING_CONFIG_FOLDER = "/conf/"

LOGGING_CONFIG_PATH = []

# This is pretty much only for development purposes.
if system() == "Windows":
    for path in [RSM_MODULE_LOCATION, DEFAULT_LOGGING_CONFIG_FOLDER]:
        LOGGING_CONFIG_PATH.append(path.replace('/', '\\'))
    LOGGING_CONFIG_PATH.append(DEFAULT_LOGGING_CONFIG_FILENAME)

LOGGING_CONFIG_ABS_PATH = "".join(i for i in LOGGING_CONFIG_PATH)


class LogHandler:
    def __init__(self, logger=__name__):
        logging.config.fileConfig(
            LOGGING_CONFIG_ABS_PATH,
            disable_existing_loggers=False)
        self.logger = logging.getLogger(logger)

    def debug(self, msg):

        return self.logger.debug(msg)

    def info(self, msg):
        return self.logger.info(msg)

    def warning(self, msg):
        return self.logger.warning(msg)

    def error(self, msg):
        return self.logger.error(msg)

    def __call__(self, fn):
        @functools.wraps(fn)
        def decorated(*args, **kwargs):
            try:
                self.logger.debug(
                    "{0} - {1} - {2}".format(fn.__name__, args, kwargs))
                result = fn(*args, **kwargs)
                self.logger.debug(result)
                return result
            except Exception as ex:
                self.logger.error("Exception {0}".format(ex))
                raise ex

        return decorated



class Entity:
    """
        This class represents an entity object.
    """

    def __init__(self):

        self.additional_properties = None

        self.modification_time = None
        self.alert_identifier = None
        self.case_identifier = None

        self.creation_time = None
        self.is_suspicious = None
        self.is_vulnerable = None

        self.entity_type = None
        self.is_internal = None
        self.is_artifact = None
        self.is_enriched = None
        self.identifier = None

        self.is_pivot = None

    @property
    def to_dict(self) -> dict:
        return self.__dict__

    def _update_internal_properties(self):
        self.additional_properties["IsEnriched"] = str(self.is_enriched)
        self.additional_properties["IsSuspicious"] = str(self.is_suspicious)
        self.additional_properties["IsVulnerable"] = str(self.is_vulnerable)
        self.additional_properties["IsInternalAsset"] = str(self.is_internal)

    def __repr__(self):
        """
        Printable representation of the object, useful for developers
        """
        return self.identifier

    def __str__(self):
        """
        Printable representation of the object, end users
        """
        return self.identifier


class Event:
    def __init__(self):
        self.modification_time = None
        self.creation_time = None
        self.identifier = None

        self.alert_identifier = None

        self.description = None
        self.event_id = None
        self.name = None

        self.device_severity = None
        self.device_product = None
        self.device_version = None
        self.event_class_id = None
        self.device_vendor = None

        self.is_correlation = None
        self.rule_generator = None
        self.event_type = None
        self.start_time = None
        self.severity = None
        self.end_time = None

        self.source_dns_domain = None
        self.device_host_name = None
        self.source_host_name = None
        self.device_address = None
        self.source_address = None

        self.source_user_name = None
        self.source_user_id = None

        self.destination_process_name = None
        self.destination_mac_address = None
        self.destination_dns_domain = None
        self.destination_nt_domain = None
        self.destination_host_name = None
        self.destination_user_name = None
        self.destination_address = None
        self.source_process_name = None
        self.source_mac_address = None
        self.destination_port = None
        self.destination_url = None

        self.file_name = None
        self.file_hash = None
        self.file_type = None

        self.application_protocol = None
        self.transport_protocol = None
        self.category_outcome = None
        self.email_subject = None

        self.additional_properties = None
        self.threat_campaign = None
        self.generic_entity = None
        self.threat_actor = None
        self.phone_number = None
        self.credit_card = None
        self.deployment = None
        self.signature = None
        self.cve = None
        self.usb = None


class AlertInfo:
    def __init__(self):

        self.alert_group_identifier = None
        self.additional_properties = None
        self.modification_time = None
        self.reporting_product = None
        self.reporting_vendor = None
        self.case_identifier = None
        self.additional_data = None
        self.rule_generator = None
        self.creation_time = None
        self.detected_time = None
        self.external_id = None
        self.description = None
        self.environment = None
        self.identifier = None
        self.severity = None
        self.name = None
        self.tags = None


class Case:
    """
        This class represents a Case object
    """

    def __init__(self):

        self.additional_properties = None
        self.modification_time = None
        self.creation_time = None
        self.identifier = None

        self.assigned_user = None
        self.is_important = None
        self.environment = None
        self.is_touched = None
        self.is_merged = None

        self.description = None
        self.priority = None
        self.status = None
        self.stage = None
        self.title = None

        self.sla_expiration_unix_time = None
        self.has_suspicious_entity = None
        self.high_risk_products = None
        self.has_workflow = None
        self.is_incident = None
        self.alert_count = None
        self.start_time = None
        self.is_locked = None


class CaseInfo:
    def __init__(self):
        self.environment = None
        self.description = None
        self.display_id = None
        self.ticket_id = None
        self.reason = None
        self.name = None
        self.device_product = None
        self.device_vendor = None
        self.start_time = None
        self.end_time = None

        self.is_test_case = False
        self.priority = -1

        self.source_grouping_identifier = None
        self.rule_generator = None
        self.attachments = []
        self.extensions = {}
        self.events = []

obj = CaseInfo()



from datetime import datetime

from rsm.handlers.output import LogHandler
from rsm.utils import misc
from rsm.utils import time

debug = LogHandler().logger.debug


class RsmUtils:
    """ Replace the original SiemplifyUtils """
    def __init__(self):
        debug('Initializing class `RsmUtils` to override methods from `SiemplifyUtils`')

    @staticmethod
    def convert_string_to_unix_time(dt_str: str) -> int:
        return time.convert_string_to_unix_time(dt_str)

    @staticmethod
    def convert_datetime_to_unix_time(dt: datetime) -> int:
        return time.convert_datetime_to_unix_time(dt, True)

    @staticmethod
    def convert_unixtime_to_datetime(epoch: int) -> datetime:
        return time.convert_unixtime_to_datetime(epoch)

    @staticmethod
    def convert_string_to_datetime(dt_str, tz=None) -> datetime:
        return time.convert_string_to_datetime(dt_str)

    @staticmethod
    def validate_timestamp(last_run_timestamp: datetime, offset: int = 0) -> datetime:
        return time.validate_timestamp(last_run_timestamp, offset)

    @staticmethod
    def dict_to_flat(d: dict) -> dict:
        """ Replace the original method `dict_to_flat` to wrap `flatten` method. """
        return misc.dict_to_flat(d)


from rsm.handlers.output import LogHandler
from rsm.override.rsmsdk import RsmSDK
import urllib3
import json

urllib3.disable_warnings()

# from SiemplifyDataModel import CyberCase, AlertInfo

log = LogHandler(__name__)

try:
	from Siemplify import Siemplify  # E402

except ModuleNotFoundError as err:
	from rsm.mocking.Siemplify import Siemplify
	log.warning("Importing mocking module `Siemplify` since the original module was not found.")
	pass
except ImportError as err:
	log.error(err)


class ApiRequest(Siemplify):
	def __init__(self):
		self.sdk_config = RsmSDK()
		super(ApiRequest, self).__init__()

	def make(self, method, endpoint, **kwargs):
		self.API_ROOT = "https://127.0.0.1/api"
		log.info(f'API_ROOT set to: {self.API_ROOT}')
		url = self.API_ROOT + endpoint
		headers = self.session.headers
		raw_data = None

		if method in ("POST", "GET"):
			if "Content-Type" not in headers:
				headers["Content-Type"] = "application/json"
				raw_data = kwargs.pop("data", {})
				raw_data = json.dumps(raw_data, sort_keys=True)
		try:
			result = self.session.request(
				method, url, raw_data, headers=headers, **kwargs)
			# if result.headers.get('Transfer-Encoding') and result.headers.get('Transfer-Encoding') == 'chunked':
			#     # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Transfer-Encoding
			#     result = result.text
			# else:
			result = result.json()
		except Exception as err:
			return err

		return result



from rsm.handlers.output import LogHandler
from rsm.override.rsmutils import RsmUtils
from rsm.override.rsmsdk import RsmSDK
from datetime import datetime
import os

log = LogHandler(__name__)
debug = log.debug
info = log.info
error = log.error

# Override the method `override_stdout` to be able to print in console.
# (Line 11, SiemplifyConnectors.py)
try:
    import SiemplifyUtils

    def _override_stdout():
        pass

    debug('Overriding method `override_stdout` with `_override_stdout`')
    SiemplifyUtils.override_stdout = _override_stdout
    SiemplifyUtils.convert_string_to_unix_time = RsmUtils.convert_string_to_unix_time
    SiemplifyUtils.convert_unixtime_to_datetime = RsmUtils.convert_unixtime_to_datetime
    SiemplifyUtils.convert_datetime_to_unix_time = RsmUtils.convert_datetime_to_unix_time
    SiemplifyUtils.convert_string_to_datetime = RsmUtils.convert_string_to_datetime
    SiemplifyUtils.dict_to_flat = RsmUtils.dict_to_flat
    # End of override, next we are importing the module and it will use the
    # override method instead of the original.
    debug('Importing module `SiemplifyBase` after overriding method')
    from SiemplifyConnectors import SiemplifyConnectorExecution  # E402

except ModuleNotFoundError as err:
    error(err)
except ImportError as err:
    error(err)


class RsmBase(SiemplifyConnectorExecution):
    def __init__(self):
        debug('Initializing class `RsmBase` with inheritance of `SiemplifyBase`')
        self.sdk_config = RsmSDK()

        # Fix: AttributeError: 'SiemplifyBase' object has no attribute
        # 'run_folder_path' (Line 13 SiemplifyBase.py)
        self.run_folder_path = self.sdk_config.run_folder_path

        info(f"'run_folder_path' = '{self.sdk_config.run_folder_path}'")
        debug('Supercharging `RsmBase`!')

        super(RsmBase, self).__init__()
        self.now = int(datetime.now().timestamp()) * 1000

    def save_timestamp(self, **kwargs):
        """
        save timestamp

        """
        new_timestamp = kwargs.get('new_timestamp', None)
        save_file_path = os.path.join(self.run_folder, self.TIMESTAMP)

        try:
            with open(save_file_path, 'w') as time_file:
                if isinstance(new_timestamp, (int, float)):
                    if len(str(int(new_timestamp))
                           ) == 10:  # timestamp in seconds
                        new_timestamp = new_timestamp * 1000
                    self.now = int(new_timestamp)
                elif new_timestamp is not None:
                    raise TypeError("new_timestamp must be in epoch time")
                time_file.write(str(self.now))
                time_file.close()

                return True
        except Exception as err:
            error(err)
            return False

    def fetch_and_save_timestamp(self, **kwargs):
        """
        fetch and save timestamp
        :return: {unix time/ datetime}
        """
        new_timestamp = kwargs.get('new_timestamp', None)

        try:
            last_run_time = self.fetch_timestamp()
            self.save_timestamp(new_timestamp=new_timestamp)
            return last_run_time
        except Exception as err:
            error(err)
            return False

    def fetch_timestamp(self, **kwargs):
        """
        get timestamp
        :param datetime_format: {boolean} if datetime - return timestamp as datetime
        :return: {unix time/ datetime}
        """
        save_file_path = os.path.join(self.run_folder, self.TIMESTAMP)
        last_run_time = 0
        if os.path.isfile(save_file_path):
            with open(save_file_path, 'r') as f:
                last_run_time = f.read()
                f.close()
        last_run_time = int(last_run_time)
        if kwargs.get('datetime_format'):
            if len(str(last_run_time)) == 13:
                last_run_time = last_run_time / 1000
            last_run_time = datetime.fromtimestamp(last_run_time)
        return last_run_time



from unittest import TestCase
from datetime import datetime
from rsm.utils import time

datetime = datetime.now()
datetime_str = "2020-06-17 23:00:00"


class TestTime(TestCase):
    def test_change_datetime_timezone_datetime(self):
        assert (type(time.from_utc_to_timezone(datetime, timezone="US/Eastern")))

    def test_change_datetime_timezone_string(self):
        assert (time.from_utc_to_timezone(datetime_str, timezone="US/Eastern"))

    def test_change_datetime_timezone_timestamp(self):
        assert time.from_utc_to_timezone(datetime.timestamp(), timezone="Europe/Amsterdam")

    def test_convert_datetime_to_unix_time(self):
        result = time.convert_datetime_to_unix_time(datetime, True)
        print(result)
        assert result

    def test_convert_string_to_datetime(self):
        result = time.convert_string_to_datetime(datetime_str)
        print(result)
        assert result

    def test_convert_unixtime_to_datetime(self):
        result = time.convert_unixtime_to_datetime(datetime.timestamp())
        print(result)
        assert result

    def test_convert_string_to_unix_time(self):
        result = time.convert_string_to_unix_time(datetime_str)
        print(result)
        assert result

    def test_validate_timestamp(self):
        result = time.validate_timestamp(datetime, offset=0)
        print(result)
        assert result





from unittest import TestCase

from datetime import \
    datetime, \
    timedelta

from rsm.override.rsmutils import RsmUtils


today = datetime.now()

#from rsm.override.rsm import RsmBase
#from rsm.override.rsmsdk import RsmSDK

# class TestClasses(TestCase):
#     def test_cls_rsmbase(self):
#         with self.assertRaises(NameError):
#             RsmBase()
#
#     def test_cls_rsmsdk(self):
#         assert RsmSDK()
#
#     def test_cls_rsmutils(self):
#         assert RsmUtils()
#


class TestRsmUtils(TestCase):
    def test_convert_string_to_unix_time(self):
        result = RsmUtils.convert_string_to_unix_time(str(today))
        print(result)
        assert result

    def test_convert_datetime_to_unix_time(self):
        result = RsmUtils.convert_datetime_to_unix_time(today)
        print(result)
        assert result

    def test_convert_string_to_datetime(self):
        result = RsmUtils.convert_string_to_datetime(str(today))
        print(result)
        assert result

    def test_convert_unixtime_to_datetime(self):
        result = RsmUtils.convert_unixtime_to_datetime(int(today.timestamp()))
        print(result)
        assert result

    def test_validate_timestamp_no_offset(self):
        self.assertEqual(today, RsmUtils.validate_timestamp(today, offset=0))

    def test_validate_timestamp_offset(self):
        self.assertEqual(first=today - timedelta(days=1),
                         second=RsmUtils.validate_timestamp(today - timedelta(days=1), offset=1))



from setuptools import setup, find_namespace_packages

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='rsm',
    version='1.2.7',
    packages=find_namespace_packages(include='rsm.*'),
    url='https://github.com/antoinethebuilder/realsiemplify',
    license='MIT',
    author='Antoine',
    author_email='64340767+antoinethebuilder@users.noreply.github.com',
    description="Let's simplify things for real, shall we? ",
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from rsm.handlers.output import LogHandler
from typing import NoReturn
import json
import os

log = LogHandler(__name__)


class EnvironmentVarNames(object):
	APP_IP = "APP_IP"


class RsmSDK:
	log.debug('The `RsmSDK` class is used to replace `SiemplifySdkConfig`')

	def __init__(self) -> NoReturn:
		current_module_path = os.path.dirname(os.path.abspath(__file__))
		sdk_config_path = os.path.join(current_module_path, "sdk_config.json")

		self.api_root_uri = "https://localhost:8443/api"
		self.config_files_root_path = "/opt/siemplify/siemplify_server/Configs"
		self.run_folder_path = "/opt/siemplify/siemplify_server/bin/Scripting"
		self.is_remote_publisher_sdk = False

		if os.path.isfile(sdk_config_path):
			try:
				with open(sdk_config_path, "r") as configFile:
					config = json.loads(configFile.read())

					# Added this line below to prevent from getting a Windows path
					# since Siemplify does not update their `sdk_config.json` file.

					if not config.get('config_files_root_path').startswith("C:"):
						log.debug("The `sdk_config` file does not start with windows-like path (e.g: C:\\)")
						self.config_files_root_path = config.get("config_files_root_path")
						self.run_folder_path = config.get("run_folder_path")

					# Determines whether or not the connector/action is running from a Remote Agent.
					self.is_remote_publisher_sdk = config.get("is_remote_publisher_sdk", False)
			except Exception as e:
				log.error("Failed to load the configuration file: " + str(e))

		self.override_with_environment_vars()

	def override_with_environment_vars(self) -> NoReturn:
		# load params from environment variables, override (give highest priority)
		simplify_app_ip_from_environment = os.environ.get(EnvironmentVarNames.APP_IP)

		if simplify_app_ip_from_environment:
			self.api_root_uri = "https://" + simplify_app_ip_from_environment + ":8443/api"


class AlertScope:
    """
        Validate and set the alert severity according to the given default levels.
    """

    def __init__(self, default_levels: list or dict, scope: str or list) -> None:

        if isinstance(scope, str):
            scope = scope.strip().split(',')

        if isinstance(default_levels, dict):
            default_levels = list(default_levels)
        elif not isinstance(default_levels, list):
            raise TypeError("Invalid type for 'default_levels'.")

        self.scope = scope
        self.default_levels = default_levels

    def is_valid_scope(self) -> bool:
        invalid = []
        try:
            for key in self.default_levels:
                for i in self.scope:
                    i = i.strip()
                    if i != key and i not in self.default_levels:
                        invalid.append(i)
            invalid = self.__remove_dup(invalid)
            if len(invalid) > 0:
                raise AttributeError(
                    f"Invalid scope parameters: {[ i.strip() for i in invalid ]}")
        except AttributeError as err:
            print(err)
            return False
        return True

    @staticmethod
    def __remove_dup(x) -> list:
        return list(dict.fromkeys(x))



import json
import traceback

EXECUTION_STATE_COMPLETED = 0
EXECUTION_STATE_INPROGRESS = 1
EXECUTION_STATE_FAILED = 2
EXECUTION_STATE_TIMEDOUT = 3


class ScriptResult:
    MAX_TOTAL_ATTACHMENT_SIZE = 5 * 1024 * 1024
    MAX_ATTACHMENT_SIZE = 3 * 1024 * 1024

    def __init__(self, entities):
        self._message = None
        self._result_value = None
        self._result_object = {}
        self._total_attachment_size = 0
        self._execution_state = EXECUTION_STATE_COMPLETED
        self._entities = entities

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def result_value(self):
        return self._result_value

    @result_value.setter
    def result_value(self, value):
        self._result_value = value

    @property
    def execution_state(self):
        return self._execution_state

    @execution_state.setter
    def execution_state(self, value):
        self._execution_state = value

    def add_entity_json(self, entity_identifier, json_data):
        """
        add json result with entity identifier as json title
        :param entity_identifier: {string} entity identifier
        :param json_data: {dict} json data
        """
        self.add_json(entity_identifier, json_data)

    def add_result_json(self, json_data):
        """
        add json result
        :param json_data: {dict} json data
        """
        # Must match Siemplify.Common.Consts.ScriptsDynamicResultFirstName
        # This is a prepation for dynamic results. In the begining, we have only one, called 'ResultJson'.
        # In the future, the names will come from the IDE Screen, like the "Output name", only dynamic
        self.add_json("JsonResult", json_data)

    def add_json(self, entity_identifier, json_data):
        """
        add json result
        :param entity_identifier: {string} entity identifier
        :param json_data: {string}/{dict}/{list} If input is json string object, it will be validated. If it's a dictionary or list, it will be json dumped
        """
        entity_data = self._get_entity_data(entity_identifier)
        if isinstance(json_data, dict) or isinstance(json_data, list):
            # In case we received a list or dict, turn it into json string
            try:
                json_data = json.dumps(json_data)
            except Exception as e:
                try:
                    # try a more lenient approach for serilization. This is not fool proof and may fail as well - str() can have errors
                    json_data = json.dumps(json_data, default=str)
                except Exception as e:
                    tb = traceback.format_exc()
                    msg = "Failed to dump json_data with default serilizier and str() as serilizier"
                    raise Exception(msg, e, tb)

        else:
            try:
                # In case we recevied a string, validate it's JSON serializable
                json.loads(json_data)
            except ValueError as err:
                raise Exception("Passed value is mot JSON serializable, Error: {0}".format(err.message))
            except TypeError as err:
                raise Exception(
                    "Expected dictionary, array or JSON serializable string, Error: {0}".format(err.message))
        entity_data["RawJson"] = json_data

    def add_entity_content(self, entity_identifier, content):
        """
        add content
        :param entity_identifier: {string} entity identifier
        :param content:
        """
        self.add_content(entity_identifier, content)

    def add_content(self, entity_identifier, content):
        """
        add content
        :param entity_identifier: {string} entity identifier
        :param content:
        """
        entity_data = self._get_entity_data(entity_identifier)
        entity_data["Content"] = content

    def add_entity_table(self, entity_identifier, data_table):
        """
        add data table with entity identifier as table title
        :param entity_identifier: {string} entity identifier
        :param data_table: {list} csv formatted list
        """
        self.add_data_table(entity_identifier, data_table)

    def add_data_table(self, title, data_table):
        """
        add data table
        :param title: {string} table title
        :param data_table: {list} csv formatted list
        """
        entity_data = self._get_entity_data(title)
        entity_data["CSVLines"] = data_table

    def add_entity_attachment(self, entity_identifier, filename, file_contents, additional_data=None):
        """
        add attachment with entity identifier as title
        :param entity_identifier: {string} entity identifier
        :param filename: {string} file name
        :param file_contents: {base64} file content
        :param additional_data:
        """
        self.add_attachment(entity_identifier, filename, file_contents, additional_data)

    def add_attachment(self, title, filename, file_contents, additional_data=None):
        """
        add attachment
        :param title: {string} attachment title
        :param filename: {string} file name
        :param file_contents: {base64} file content
        :param additional_data:
        :return:
        """
        self._validate_attachment_size(len(file_contents))

        file_dict = {}
        if additional_data is not None:
            file_dict.update(additional_data)

        file_dict["filename"] = filename
        file_dict["file_contents"] = file_contents

        entity_data = self._get_entity_data(title)
        entity_data["Attachments"][filename] = file_contents
        self._total_attachment_size += len(file_contents)

    def add_entity_html_report(self, entity_identifier, report_name, report_contents):
        """
        add html data with entity identifier as title
        :param entity_identifier: {string} entity identifier
        :param report_name:{string} html report name
        :param report_contents: html content
        :return:
        """
        self.add_html(entity_identifier, report_name, report_contents)

    def add_html(self, title, report_name, report_contents):
        """
        add html data
        :param title: {string} title
        :param report_name: {string} html report name
        :param report_contents: html content
        """
        self._validate_attachment_size(len(report_contents))

        entity_data = self._get_entity_data(title)
        entity_data["Htmls"][report_name] = report_contents

    def add_entity_link(self, entity_identifier, link):
        """
        add web link with entity identifier as title
        :param entity_identifier: {string} entity identifier
        :param link: {string} link
        """
        self.add_link(entity_identifier, link)

    def add_link(self, title, link):
        """
        add web link
        :param title: {string} link title
        :param link: {string} link
        """
        entity_data = self._get_entity_data(title)
        if "Links" not in entity_data:
            entity_data["Links"] = []
        entity_data["Links"].append(link)

    def _get_entity_data(self, title):
        """
        get entity data
        :param title: {string} entity identifier
        :return: entity data
        """
        if title not in self._result_object:
            is_entity = False
            if title in self._entities:
                is_entity = True
            self._result_object[title] = {"Title": title,
                                          "IsForEntity": is_entity,
                                          "Content": "",
                                          "RawJson": "",
                                          "CSVLines": "",
                                          "Attachments": {},
                                          "Htmls": {}}
        return self._result_object[title]

    def _validate_attachment_size(self, attachment_size):
        """
        Validate attachment size - limit to 5 MB.
        :param attachment_size: {int} attachment file size
        """
        if attachment_size > self.MAX_ATTACHMENT_SIZE:
            raise EnvironmentError(
                "Attachment cannot be larger than %d MB" % int(self.MAX_ATTACHMENT_SIZE / 1024 / 1024))
        if attachment_size + self._total_attachment_size > self.MAX_TOTAL_ATTACHMENT_SIZE:
            raise EnvironmentError("Total entity attachments cannot be larger than %d MB" % int(
                self.MAX_TOTAL_ATTACHMENT_SIZE / 1024 / 1024))
